@@ -28,6 +28,29 @@ sub hello : Global {
   $context->response->output($html);
 }
 
+# absolute redirect
+sub hi : Global {
+  my($self, $context) = @_;
+  my $where = $context->uri_for('hello');
+  $context->response->redirect( $where );
+  return;
+}
+
+# partial (relative) redirect
+sub greetings : Global {
+  my($self, $context) = @_;
+  $context->response->redirect( "hello" );
+  return;
+}
+
+# redirect to a redirect
+sub bonjour : Global {
+  my($self, $context) = @_;
+  my $where = $context->uri_for('hi');
+  $context->response->redirect( $where );
+  return;
+}
+
 sub html {
   my($title, $body) = @_;
   return qq{
